﻿Option Explicit On
'Open notes section - last updated 5 August 2016
'Add frequency entry / query single box.
'Alter measure query results as displayed in text boxes to represent 'reasonable' precision. Maybe out to 4 decimal places.
'Zero and cal addition potentially or not that it does not exist in application.
'Existing order / re-order sub occurrences in main. Not critical, but better orginazation would be nice.
'More robust error trapping and testing;
'Spelling checks;

Public Class Form1
    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load

        'On form load do not allow measure or fetch query via associated buttons 
        'until the connection button has been successfully executed.
        setAcquisitionButtonEnableMode(False)

        'On form load do not show Acquiring Data Label
        setAcquiringDataLblMode(False)

        'On form load do not allow selection of Units without prior instrument connection
        unitDbmRadBtn.Enabled = False
        unitWattsRadBtn.Enabled = False

        'On form load engage tooltips
        setupToolTips()

    End Sub

    Dim rscMngr As Ivi.Visa.Interop.ResourceManager
    Dim pwrMtr As Ivi.Visa.Interop.FormattedIO488
    Dim status As Ivi.Visa.Interop.VisaStatusCode
    Dim ex As Exception
    Dim errMessageText As String
    Dim myStopWatch As New System.Diagnostics.Stopwatch
    Dim fetchComplete, done As Int16
    Dim measUnit As String = ""


    'A sub to exit / close / shut down the application
    Private Sub exitBtn_Click(sender As Object, e As EventArgs) Handles exitBtn.Click
        'Allow the user to exit without ever connecting to a valid instrument and exit gracefully
        If rscMngr Is Nothing Then
            End
        Else
            pwrMtr.IO.Close()
            End

        End If
    End Sub

    'This sub connects the targeted power meter / power sensor.
    'Successful connection is reliant on a valid VISA resource string as provided or obtained 
    'via the installed I/O libraries, such as Keysight I/O Libs version xx. 
    Private Sub connectBtn_Click(sender As Object, e As EventArgs) Handles connectBtn.Click
        'If there is some information in the idOutputTextBox then clear it
        'idOutputTxtBox.
        'On Error GoTo ioErrHandler

        'On connect and / or reconnect do not allow measure or fetch query via associated buttons 
        'until the connection button has been successfully executed.
        setAcquisitionButtonEnableMode(False)
        Try
            rscMngr = New Ivi.Visa.Interop.ResourceManager
            pwrMtr = New Ivi.Visa.Interop.FormattedIO488
            errMessageText = New Ivi.Visa.Interop.VisaStatusCode

            pwrMtr.IO = rscMngr.Open(ioAddressTxtBox.Text)

            'Alter defual time out from 2000mS to 5000mS
            pwrMtr.IO.Timeout = 5000

            pwrMtr.WriteString("*IDN?", True)
            idOutputTxtBox.Text = pwrMtr.ReadString()

            'Upon power meter / sensor connection confirmation reset to default with *OPC? operation complete query return
            pwrMtr.WriteString("SYST:PRES DEF;*OPC?", True)
            done = pwrMtr.ReadString()

            'Also, on reset query units and set / update units Radio button selection
            pwrMtr.WriteString("UNIT:POW?", True)
            measUnit = pwrMtr.ReadString()

            'Enable units radio button once connected and set dBm radio button to checked
            unitDbmRadBtn.Enabled = True
            unitDbmRadBtn.Checked = True
            unitWattsRadBtn.Enabled = True


            'Based on result of units query ensure correct radio button is set 
            If UCase(measUnit).Contains("DBM") Then
                unitDbmRadBtn.Checked = (True)
                measUnit = "dBm"
            Else
                'A bit verbose as the radio button can only be one or the other. 
                unitWattsRadBtn.Checked = (True)
                measUnit = "Watts"
            End If


            'If the above worked then connection has been established 
            setAcquisitionButtonEnableMode(True)

        Catch ex As Exception

            MsgBox("Error occured: " & Err.Description)

        Finally

        End Try

        'More work here 4Aug2016
        'ioErrHandler:
        'MsgBox("Connection / Indenfiication Error: " & vbCrLf & Err.Description & " more stuff")

    End Sub

    Private Sub measQueryBtn_Click(sender As Object, e As EventArgs) Handles measQueryBtn.Click

        'Form management first 

        'Manage details surrounding indicated text boxes. Clear contents on meas execute as acquisition time could be quite long as a function of RF input level.
        clearMeasText()

        'On measure show Acquring Data Label
        setAcquiringDataLblMode(True)

        'On measure entry do not allow measure or fetch query till the connection button has been successfully executed.
        setAcquisitionButtonEnableMode(False)

        'Alter the default timeout from 5s to 60s. The "MEASure?" query command invokes the auto average a.k.a. filter length. For singnals
        'which are near the low level / noise level of the sensor the average count may be the maximum value of 1024 averages. This maximum 
        'average count will result in an extremely long acquisition time period of upt to 45 seconds or longer. On exit of this sub return
        'the timeout to a reasonable 5 seconds.
        pwrMtr.IO.Timeout = 60000

        'Initiate the stopwatch so that approximate aquisition time can be displayed / computed. 
        myStopWatch.Reset()
        myStopWatch.Start()
        '5Aug2016. Appears as though a bug in the power meter firmware?
        'The MEAS? query should set the average count automatically first time based on RF input level.
        'However, the manual average count is persisting regardless of level. For the second measure query
        'the auto average count is getting set as one would expect. 
        'Assertion of the command SENS:AVER:COUN:AUTO is an attempt to correct this issue.
        'Command below disabled as this did not fix this issue - TS 5Aug2016
        'pwrMtr.WriteString("SENS:AVER:COUN:AUTO ON", True)

        'Query the measurement of the RF power level / value
        pwrMtr.WriteString("MEAS?", True)
        measValueResultTxtBox.Text = pwrMtr.ReadString() & measUnit

        'Stop stopwatch
        myStopWatch.Stop()

        'Set Acquire time window 
        Dim timeElasped As TimeSpan = myStopWatch.Elapsed
        Dim timeResults As String = Nothing
        timeResults = String.Format("{0} seconds : " & "{1} milliseconds", timeElasped.Seconds, timeElasped.Milliseconds)
        measTimeResultTxtBox.Text = timeResults

        'Query the average count of sensor. The MEASure? query forces auto averaging. 
        pwrMtr.WriteString("SENS:AVER:COUNT?", True)
        avgCntMeasQueryTxtBox.Text = pwrMtr.ReadString()

        'On measure exit do not show Acquring Data Label
        setAcquiringDataLblMode(False)

        'On measure exit allow or enable both measure and fetch query buttons
        setAcquisitionButtonEnableMode(True)

        'On measure exit return timeout to a reasonable 5 seconds.
        pwrMtr.IO.Timeout = 5000

    End Sub

    'A sub to clear the text boxes within / related to the MEASure? query results window.
    Private Sub clearMeasText()

        measTimeResultTxtBox.Clear()
        measTimeResultTxtBox.Update()

        measValueResultTxtBox.Clear()
        measValueResultTxtBox.Update()

        avgCntMeasQueryTxtBox.Clear()
        avgCntMeasQueryTxtBox.Update()

    End Sub

    'A sub to clear the text boxes within / related to the FETCh? query results window.
    Private Sub clearFetchText()
        fetchTimeTxtBox.Clear()
        fetchTimeTxtBox.Update()

        fetchValueResultTxtBox.Clear()
        fetchValueResultTxtBox.Update()

        avgCntFetchQueryTxtBox.Clear()
        avgCntFetchQueryTxtBox.Update()

    End Sub


    'A sub to either enable or disable the meas and fetch acquire buttons
    Private Sub setAcquisitionButtonEnableMode(mode As Boolean)
        measQueryBtn.Enabled = mode
        fetchQueryBtn.Enabled = mode
    End Sub

    'A sub to either enable or disable the acquiring data label
    Private Sub setAcquiringDataLblMode(acqMode As Boolean)
        acquiringDataLbl.Visible = acqMode
        acquiringDataLbl.Update()

    End Sub

    'An instrument error check routine. Leave in body but for debug only.
    Private Sub instrumentErrCheck()
        Dim errResult As String = "9E99"

        'Query instrument error queue
        pwrMtr.WriteString("SYST:ERR?", True)
        errResult = pwrMtr.ReadString()

        If Not (UCase(errResult).Contains("NO ERROR")) Then
            MsgBox("Instrument Error " & errResult, MsgBoxStyle.OkCancel)
        End If

    End Sub

    Private Sub Button1_Click(sender As Object, e As EventArgs) Handles instErrCheckBtn.Click
        instrumentErrCheck()
    End Sub

    Private Sub senseUnitsChanged()

        'If the units is altered reset all displayed acquire time, measure and average count to ensure new measurement is required.
        clearFetchText()
        clearMeasText()

        'Also set the units on the power meter
        If unitDbmRadBtn.Checked = True Then
            pwrMtr.WriteString("UNIT:POW DBM", True)
            measUnit = "dBm"
        Else
            pwrMtr.WriteString("UNIT:POW W", True) '
            measUnit = "Watts"
        End If
    End Sub

    'A sub to sense the units have been changed via unitWattBtn change
    Private Sub unitWattsRadBtn_CheckedChanged(sender As Object, e As EventArgs) Handles unitWattsRadBtn.CheckedChanged
        senseUnitsChanged()
    End Sub


    'A sub that handles the Fetch Query button. User can enter / alter the average count
    Private Sub fetchQueryBtn_Click(sender As Object, e As EventArgs) Handles fetchQueryBtn.Click
        fetchQueryInvoke()

    End Sub
    '###TomSilchia 5Aug2016 this could use a bit better integration. Must double mouse click to update via user entered FETCh? box.
    'With defualt on value changed then on FORM load the value is changing, instrument CONNECT has not happened, and VB.NET system
    'error occurs. Maybe able to disable the user entry FETCh? query area until the user has entered this area first time then enable.
    Private Sub fetchAvgCntSetTxtBx_TextChanged(sender As Object, e As EventArgs) Handles fetchAvgCntSetTxtBx.DoubleClick 'MouseDoubleClick

        fetchQueryInvoke()
    End Sub
    Private Sub fetchQueryInvoke()
        'Form management first 

        'Manage details surrounding indicated text boxes. Clear contents on meas execute as aquisition time could be quite long as a function of RF input level.
        clearFetchText()

        'On fetch show Acquring Data Label
        setAcquiringDataLblMode(True)

        'On fetch entry do not allow measure or fetch query till the connection button has been successfully executed.
        setAcquisitionButtonEnableMode(False)

        'Initiate the stopwatch so that approximate aquisition time can be displayed / computed. 
        myStopWatch.Reset()
        myStopWatch.Start()

        'Alter the default timeout from 5s to 60s. The "MEASure?" query command invokes the auto average a.k.a. filter length. For singnals
        'which are near the low level / noise level of the sensor the average count may be the maximum value of 1024 averages. This maximum 
        'average count will result in an extremely long acquisition time period of up to 45 seconds or longer. On exit of this sub return
        'the timeout to a reasonable 5 seconds.
        pwrMtr.IO.Timeout = 60000

        'Apply the user settable fetch average count to the power meter FETCh? average count 
        pwrMtr.WriteString("SENS:AVER:COUNT " & fetchAvgCntSetTxtBx.Text, True)

        pwrMtr.WriteString("SENS:AVER:COUNT?", True)
        avgCntFetchQueryTxtBox.Text = pwrMtr.ReadString

        'Query via FETCh? the RF power level / value
        'Set trigger mode to hold
        pwrMtr.IO.WriteString("INIT:CONT OFF")


        'Initiate the stopwatch so that approximate aquisition time can be displayed / computed. 
        myStopWatch.Reset()
        myStopWatch.Start()

        'Assert Trigger with hold off
        pwrMtr.IO.WriteString("INIT:IMM;*OPC?")
        fetchComplete = pwrMtr.ReadString()
        pwrMtr.IO.WriteString("FETCh?")
        fetchValueResultTxtBox.Text = pwrMtr.ReadString() & measUnit

        'Set Acquire time window 
        myStopWatch.Stop()
        Dim timeElasped As TimeSpan = myStopWatch.Elapsed
        Dim timeResults As String = Nothing
        timeResults = String.Format("{0} seconds : " & "{1} milliseconds", timeElasped.Seconds, timeElasped.Milliseconds)
        fetchTimeTxtBox.Text = timeResults

        'Set trigger mode to freeRun ###TS 5Aug2016 @ 1330.
        pwrMtr.IO.WriteString("INIT:CONT ON")

        'On measure exit do not show Acquring Data Label
        setAcquiringDataLblMode(False)

        'On measure exit allow or enable both measure and fetch query buttons
        setAcquisitionButtonEnableMode(True)
    End Sub

    Sub setupToolTips()
        Dim toolTipAutoPopDelay As Int16 = 4000
        Dim toolTipInitialDelay As Int16 = 400
        Dim toolTipReShowlDelay As Int16 = 4000


        'Tooltip for the ioAddressTxtBox
        Dim ioAddressTxtBoxTip As New ToolTip
        ioAddressTxtBoxTip.ShowAlways = True
        ioAddressTxtBoxTip.IsBalloon = True
        ioAddressTxtBoxTip.UseAnimation = True
        ioAddressTxtBoxTip.UseAnimation = True
        ioAddressTxtBoxTip.AutoPopDelay = toolTipAutoPopDelay
        ioAddressTxtBoxTip.InitialDelay = toolTipInitialDelay
        ioAddressTxtBoxTip.ReshowDelay = toolTipReShowlDelay
        ioAddressTxtBoxTip.SetToolTip(ioAddressTxtBox, "Insert a VISA resource string as provided via the I/O libraries" _
                                      & vbLf & "For example: 'GPIP0::13::INSTR'")

        'Tooltip for the Connect Button
        Dim connectBtnTip As New ToolTip
        connectBtnTip.ShowAlways = True
        connectBtnTip.IsBalloon = True
        connectBtnTip.UseAnimation = True
        connectBtnTip.UseAnimation = True
        connectBtnTip.AutoPopDelay = toolTipAutoPopDelay
        connectBtnTip.InitialDelay = toolTipInitialDelay
        connectBtnTip.ReshowDelay = toolTipReShowlDelay
        connectBtnTip.SetToolTip(connectBtn, "Once a valid VISA resource string has been set " & vbLf &
                                             "select Connect to initate communications.")

        'Tooltip for the MEASure? aquisition Button
        Dim measQueryBtnTip As New ToolTip
        measQueryBtnTip.ShowAlways = True
        measQueryBtnTip.IsBalloon = True
        measQueryBtnTip.UseAnimation = True
        measQueryBtnTip.UseAnimation = True
        measQueryBtnTip.AutoPopDelay = 24000 'toolTipAutoPopDelay
        measQueryBtnTip.InitialDelay = toolTipInitialDelay
        measQueryBtnTip.ReshowDelay = toolTipReShowlDelay
        measQueryBtnTip.SetToolTip(measQueryBtn,
                                   "The measure query button invokes a 'MEASure?' query to the target power meter or sensor." & vbLf &
                                   "The 'MEAS?' query provides the simplest way to program a power sensor for measurements." & vbLf &
                                   "This is a compound command equivalent to an ABORT followed by a CONFigure and a READ?. " & vbLf &
                                   "It does not enable much flexibility or control over measurement settings. " & vbLf & vbLf &
                                   "The power sensor selects the best settings for the requested configuration" & vbLf &
                                   "and immediately performs the measurement. This means you cannot fine tune " & vbLf &
                                   "the measurement, for example, you cannot change the filter length i.e. the" & vbLf &
                                   "the 'SENS:AVER:COUNT:AUTO ON' is enabled. Due to the auto average / filter length" & vbLf &
                                   "Low RF power levels may require 1024 averages resulting in up to 45 second acquisition times.")

        'Tooltip for the FETCh? aquisition Button
        Dim fetchQueryBtnTip As New ToolTip
        fetchQueryBtnTip.ShowAlways = True
        fetchQueryBtnTip.IsBalloon = True
        fetchQueryBtnTip.UseAnimation = True
        fetchQueryBtnTip.UseAnimation = True
        fetchQueryBtnTip.AutoPopDelay = 24000 'toolTipAutoPopDelay
        fetchQueryBtnTip.InitialDelay = toolTipInitialDelay
        fetchQueryBtnTip.ReshowDelay = toolTipReShowlDelay
        fetchQueryBtnTip.SetToolTip(fetchQueryBtn,
                                 "The fetch query button invokes a 'FETCh?' query to the target power meter or sensor." & vbLf &
                                 "Utilizing the 'FETCh?' query command allows the user advance control of the power meter" & vbLf &
                                 "such as manually setting the average count / filter length via the 'SENSe:AVERage:COUNt n'." & vbLf &
                                 "This manual average count has been implemented via use of 'FETCh?' command. " & vbLf & vbLf &
                                 "Use of 'FETCh?' allows low level RF signals to be acquired fast, at potentially" & vbLf &
                                 "degraded repeatability.'FETCh? requires use of a TRIGGER where as 'MEASure?'does not.")
    End Sub
End Class

